package com.fjhx.service.impl;

import cn.hutool.core.collection.ConcurrentHashSet;
import com.alibaba.fastjson.JSONObject;
import com.fjhx.entity.ConnectConfig;
import com.fjhx.service.Operation;
import com.fjhx.utils.MyUtil;
import com.fjhx.utils.ThreadPoolManager;
import com.rfid.callBack.CallBack;
import com.rfid.uhf.controller.impl.ReaderR2000;
import com.rfid.uhf.service.ReaderR2000Service;
import com.rfid.uhf.service.impl.ReaderR2000ServiceImpl;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

public class TcpOperation implements Operation {

    private final ReaderR2000Service service = new ReaderR2000ServiceImpl();
    private final List<ReaderR2000> readerR2000List = new ArrayList<>();
    private final Set<String> rfidSet = new ConcurrentHashSet<>();

    @Override
    public void distinctRead(String userId, String sessionId, List<ConnectConfig> connectConfigList, JSONObject data) {

        for (ConnectConfig connectConfig : connectConfigList) {
            ThreadPoolManager.execute(() -> {

                ReaderR2000 reader = service.connect(
                        connectConfig.getIp(),
                        connectConfig.getPort(),
                        getCallBack(userId, sessionId, data)
                );

                service.beginInvV2(reader);
                readerR2000List.add(reader);

            });
        }

    }

    @Override
    public void close() {
        for (ReaderR2000 readerR2000 : readerR2000List) {
            // 结束扫描
            service.stopInvV2(readerR2000);
            // 关闭连接
            service.disconnect(readerR2000);
        }
    }

    private CallBack.R2000 getCallBack(String userId, String sessionId, JSONObject data) {
        return new CallBack.R2000() {

            @Override
            public void readData(String rfid, String rssi, String antennaNo, String deviceNo, String direction, String communicationMode) {
                if (rfidSet.contains(rfid)) {
                    return;
                }
                rfidSet.add(rfid);

                MyUtil.infoLog("扫描到rfid标签:  userId:" + userId + ", sessionId:" + sessionId + ", instructions:"
                        + data.getString("instructions") + ", rfid:" + rfid);
                sendRfidDate(userId, sessionId, rfid, data);
            }

            public void serialPortException(String name, boolean result) {

            }

            public void heartBeat(String value, String deviceNo, String communicationMode) {
                // System.out.println("value " + value + " deviceNo " + deviceNo + " communicationMode " + communicationMode);
            }

            public void infraredTrigger(boolean direction, String value, String deviceNo, String communicationMode) {
                // System.out.println("方向 " + direction + " 红外值 " + value + " 设备号 " + deviceNo + " IP或串口号 " + communicationMode);
            }

            public void onceReadTagEnd(boolean arg0) {

            }

        };
    }

}
